<?php

function _member_mailman_add_base_form(&$form) {
  $form['email'] = array(
     '#type' => 'textfield',
     '#title' => t('E-mail address'),
     '#description' => t('A confirmation message will be sent to this address with further instructions.'),
     '#required' => TRUE,
     '#default_value' => '',
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#weight' => 100,
  );
  $form['new'] = array(
     '#type' => 'value',
     '#value' => FALSE,
  );
  $form['#submit'] = array('member_mailman_op_form_submit');
  $form['#redirect'] = '<front>';
}

function member_mailman_op_form_submit($form, &$form_state) {
  member_mailman_send_confirmation($form_state['values']);
  drupal_set_message(t('Further instructions have been sent to your e-mail address.'));
}

function member_mailman_subscribe_form($form_state) {
  $form = array();
  _member_mailman_add_base_form($form);
  $form['first_name'] = array(
     '#type' => 'textfield',
     '#title' => t('First name'),
     '#required' => TRUE,
     '#default_value' => '',
  );
  $form['last_name'] = array(
     '#type' => 'textfield',
     '#title' => t('Last name'),
     '#required' => TRUE,
     '#default_value' => '',
  );
  $form['address'] = array(
     '#type' => 'textfield',
     '#title' => t('Address'),
     '#required' => TRUE,
     '#default_value' => '',
  );
  $form['pending_operation'] = array(
     '#type' => 'value',
     '#value' => 'subscribe',
  );
  return $form;
}

function member_mailman_subscribe_form_validate($form, &$form_state) {
  if (!valid_email_address($form_state['values']['email'])) {
    form_set_error('email', t('Invalid e-mail address format'));
  }
  else {
    $existing = db_fetch_array(db_query("SELECT email, pending_operation FROM {member_mailman} WHERE email = LOWER('%s')", $form_state['values']['email']));
    if ($existing) {
      if ($existing['pending_operation'] != 'subscribe') {
        // We can re-send the subscribe confirmation, but otherwise there is an error.
        form_set_error('email', t('This e-email address is already subscribed.'));
      }
    }
    else {
      $form_state['values']['new'] = TRUE;
    }
  }
}

function member_mailman_change_form($form_state) {
  $form = array();
  _member_mailman_add_base_form($form);
  $form['new_email'] = array(
     '#type' => 'textfield',
     '#title' => t('New E-mail address'),
     '#required' => TRUE,
     '#default_value' => '',
  );
  $form['pending_operation'] = array(
     '#type' => 'value',
     '#value' => 'change',
  );
  return $form;
}

function member_mailman_change_form_validate($form, &$form_state) {
  if (!valid_email_address($form_state['values']['email'])) {
    form_set_error('email', t('Invalid e-mail address format'));
  }
  elseif (!valid_email_address($form_state['values']['new_email'])) {
    form_set_error('new_email', t('Invalid e-mail address format'));
  }
  else {
    $existing = db_fetch_array(db_query("SELECT email, pending_operation FROM {member_mailman} WHERE email = LOWER('%s')", $form_state['values']['email']));
    if (!$existing) {
      form_set_error('email', t('This e-mail address is not currently subscribed'));
    }
  }
}

function member_mailman_remove_form($form_state) {
  $form = array();
  _member_mailman_add_base_form($form);
  $form['pending_operation'] = array(
     '#type' => 'value',
     '#value' => 'remove',
  );
  return $form;
}

function member_mailman_remove_form_validate($form, &$form_state) {
  if (!valid_email_address($form_state['values']['email'])) {
    form_set_error('email', t('Invalid e-mail address format'));
  }
  else {
    $existing = db_fetch_array(db_query("SELECT email, pending_operation FROM {member_mailman} WHERE email = LOWER('%s')", $form_state['values']['email']));
    if (!$existing) {
      form_set_error('email', t('This e-mail address is not currently subscribed'));
    }
  }
}

function member_mailman_confirm_form($form_state, $op, $email) {
  $form = array();
  $form['pending_operation'] = array(
    '#type' => 'value',
    '#value' => $op,
  );
  $form['email'] = array(
    '#type' => 'value',
    '#value' => $email,
  );
  $form['#redirect'] = '<front>';
  $path = '<front>';
  switch ($op) {
    case 'subscribe':
      $question = 'Confirm e-mail list subscription for %email?';
      $description = t('You will be added to our announcements list.');
      break;
    case 'change':
      $question = 'Confirm e-mail list address change for %email?';
      $new_email = db_result(db_query("SELECT new_email FROM {member_mailman} WHERE email = '%s'", $email));
      $description = t('%new_email will be used instead for our announcements list.', array('%new_email' => $new_email));
      break;
    case 'remove':
      $question = 'Confirm e-mail list address removal for %email?';
      $description = t('Your will be removed from our announcements list.');
      break;
  }
  return confirm_form($form, t($question, array('%email' => $email)), $path, $description);
}

function member_mailman_confirm_form_submit($form, &$form_state) {
  db_query("UPDATE {member_mailman} SET operation_confirmed = 1, token = '' WHERE email = '%s' AND pending_operation = '%s'", $form_state['values']['email'], $form_state['values']['pending_operation']);
  drupal_set_message('Your request has been confirmed and will be completed shortly.');
}
